@@ -26,16 +26,15 @@ import functools
 import sys
 
 from keystoneclient import exceptions as keystone_exceptions
+from oslo_config import cfg
 from oslo_log import log
 import six
 
 from watcher._i18n import _
 
-from watcher import conf
-
 LOG = log.getLogger(__name__)
 
-CONF = conf.CONF
+CONF = cfg.CONF
 
 
 def wrap_keystone_exception(func):
@@ -122,6 +121,11 @@ class NotAuthorized(WatcherException):
     code = 403
 
 
+class NotAcceptable(WatcherException):
+    msg_fmt = _("Request not acceptable.")
+    code = 406
+
+
 class PolicyNotAuthorized(NotAuthorized):
     msg_fmt = _("Policy doesn't allow %(action)s to be performed.")
 
@@ -206,6 +210,10 @@ class InvalidIntervalOrCron(Invalid):
     msg_fmt = _("Expected an interval or cron syntax but received %(name)s")
 
 
+class DataModelTypeNotFound(ResourceNotFound):
+    msg_fmt = _("The %(data_model_type)s data model could not be found")
+
+
 class GoalNotFound(ResourceNotFound):
     msg_fmt = _("Goal %(goal)s could not be found")
 
@@ -231,11 +239,6 @@ class AuditTemplateAlreadyExists(Conflict):
                 "already exists")
 
 
-class AuditTemplateReferenced(Invalid):
-    msg_fmt = _("AuditTemplate %(audit_template)s is referenced by one or "
-                "multiple audits")
-
-
 class AuditTypeNotFound(Invalid):
     msg_fmt = _("Audit type %(audit_type)s could not be found")
 
@@ -260,6 +263,11 @@ class AuditIntervalNotAllowed(Invalid):
     msg_fmt = _("Interval of audit must not be set for %(audit_type)s.")
 
 
+class AuditStartEndTimeNotAllowed(Invalid):
+    msg_fmt = _("Start or End time of audit must not be set for "
+                "%(audit_type)s.")
+
+
 class AuditReferenced(Invalid):
     msg_fmt = _("Audit %(audit)s is referenced by one or multiple action "
                 "plans")
@@ -350,38 +358,10 @@ class IllegalArgumentException(WatcherException):
     msg_fmt = _('Illegal argument')
 
 
-class NoSuchMetric(WatcherException):
-    msg_fmt = _('No such metric')
-
-
-class NoDataFound(WatcherException):
-    msg_fmt = _('No rows were returned')
-
-
 class AuthorizationFailure(WatcherException):
     msg_fmt = _('%(client)s connection failed. Reason: %(reason)s')
 
 
-class KeystoneFailure(WatcherException):
-    msg_fmt = _("Keystone API endpoint is missing")
-
-
-class ClusterEmpty(WatcherException):
-    msg_fmt = _("The list of compute node(s) in the cluster is empty")
-
-
-class ComputeClusterEmpty(WatcherException):
-    msg_fmt = _("The list of compute node(s) in the cluster is empty")
-
-
-class StorageClusterEmpty(WatcherException):
-    msg_fmt = _("The list of storage node(s) in the cluster is empty")
-
-
-class MetricCollectorNotDefined(WatcherException):
-    msg_fmt = _("The metrics resource collector is not defined")
-
-
 class ClusterStateStale(WatcherException):
     msg_fmt = _("The cluster state is stale")
 
@@ -394,10 +374,6 @@ class ClusterStateNotDefined(WatcherException):
     msg_fmt = _("The cluster state is not defined")
 
 
-class CapacityNotDefined(WatcherException):
-    msg_fmt = _("The capacity %(capacity)s is not defined for '%(resource)s'")
-
-
 class NoAvailableStrategyForGoal(WatcherException):
     msg_fmt = _("No strategy could be found to achieve the '%(goal)s' goal.")
 
@@ -412,10 +388,6 @@ class GlobalEfficacyComputationError(WatcherException):
                 "goal using the '%(strategy)s' strategy.")
 
 
-class NoMetricValuesForInstance(WatcherException):
-    msg_fmt = _("No values returned by %(resource_id)s for %(metric_name)s.")
-
-
 class UnsupportedDataSource(UnsupportedError):
     msg_fmt = _("Datasource %(datasource)s is not supported "
                 "by strategy %(strategy)s")
@@ -425,6 +397,16 @@ class DataSourceNotAvailable(WatcherException):
     msg_fmt = _("Datasource %(datasource)s is not available.")
 
 
+class MetricNotAvailable(WatcherException):
+    """Indicate that a metric is not configured or does not exists"""
+    msg_fmt = _('Metric: %(metric)s not available')
+
+
+class NoDatasourceAvailable(WatcherException):
+    """No datasources have been configured"""
+    msg_fmt = _('No datasources available')
+
+
 class NoSuchMetricForHost(WatcherException):
     msg_fmt = _("No %(metric)s metric for %(host)s found.")
 
@@ -469,6 +451,11 @@ class InstanceNotFound(ComputeResourceNotFound):
     msg_fmt = _("The instance '%(name)s' could not be found")
 
 
+class InstanceNotMapped(ComputeResourceNotFound):
+    msg_fmt = _("The mapped compute node for instance '%(uuid)s' "
+                "could not be found.")
+
+
 class ComputeNodeNotFound(ComputeResourceNotFound):
     msg_fmt = _("The compute node %(name)s could not be found")
 
