@@ -16,37 +16,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-"""
-[PoC]Uniform Airflow using live migration
 
-*Description*
-
-It is a migration strategy based on the airflow of physical
-servers. It generates solutions to move VM whenever a server's
-airflow is higher than the specified threshold.
-
-*Requirements*
-
-* Hardware: compute node with NodeManager 3.0 support
-* Software: Ceilometer component ceilometer-agent-compute running
-  in each compute node, and Ceilometer API can report such telemetry
-  "airflow, system power, inlet temperature" successfully.
-* You must have at least 2 physical compute nodes to run this strategy
-
-*Limitations*
-
-- This is a proof of concept that is not meant to be used in production.
-- We cannot forecast how many servers should be migrated. This is the
-  reason why we only plan a single virtual machine migration at a time.
-  So it's better to use this algorithm with `CONTINUOUS` audits.
-- It assumes that live migrations are possible.
-"""
-
-from oslo_config import cfg
 from oslo_log import log
 
 from watcher._i18n import _
-from watcher.common import exception as wexc
 from watcher.decision_engine.model import element
 from watcher.decision_engine.strategy.strategies import base
 
@@ -84,27 +57,6 @@ class UniformAirflow(base.BaseStrategy):
 
     DATASOURCE_METRICS = ['host_airflow', 'host_inlet_temp', 'host_power']
 
-    METRIC_NAMES = dict(
-        ceilometer=dict(
-            # The meter to report Airflow of physical server in ceilometer
-            host_airflow='hardware.ipmi.node.airflow',
-            # The meter to report inlet temperature of physical server
-            # in ceilometer
-            host_inlet_temp='hardware.ipmi.node.temperature',
-            # The meter to report system power of physical server in ceilometer
-            host_power='hardware.ipmi.node.power'),
-        gnocchi=dict(
-            # The meter to report Airflow of physical server in gnocchi
-            host_airflow='hardware.ipmi.node.airflow',
-            # The meter to report inlet temperature of physical server
-            # in gnocchi
-            host_inlet_temp='hardware.ipmi.node.temperature',
-            # The meter to report system power of physical server in gnocchi
-            host_power='hardware.ipmi.node.power'),
-    )
-
-    MIGRATION = "migrate"
-
     def __init__(self, config, osc=None):
         """Using live migration
 
@@ -115,12 +67,6 @@ class UniformAirflow(base.BaseStrategy):
         super(UniformAirflow, self).__init__(config, osc)
         # The migration plan will be triggered when the airflow reaches
         # threshold
-        self.meter_name_airflow = self.METRIC_NAMES[
-            self.config.datasource]['host_airflow']
-        self.meter_name_inlet_t = self.METRIC_NAMES[
-            self.config.datasource]['host_inlet_temp']
-        self.meter_name_power = self.METRIC_NAMES[
-            self.config.datasource]['host_power']
         self._period = self.PERIOD
 
     @classmethod
@@ -180,16 +126,6 @@ class UniformAirflow(base.BaseStrategy):
             },
         }
 
-    @classmethod
-    def get_config_opts(cls):
-        return [
-            cfg.StrOpt(
-                "datasource",
-                help="Data source to use in order to query the needed metrics",
-                default="gnocchi",
-                choices=["ceilometer", "gnocchi"])
-        ]
-
     def get_available_compute_nodes(self):
         default_node_scope = [element.ServiceState.ENABLED.value]
         return {uuid: cn for uuid, cn in
@@ -221,18 +157,20 @@ class UniformAirflow(base.BaseStrategy):
             source_instances = self.compute_model.get_node_instances(
                 source_node)
             if source_instances:
-                inlet_t = self.datasource_backend.statistic_aggregation(
-                    resource_id=source_node.uuid,
-                    meter_name=self.meter_name_inlet_t,
+                inlet_temp = self.datasource_backend.statistic_aggregation(
+                    resource=source_node,
+                    resource_type='instance',
+                    meter_name='host_inlet_temp',
                     period=self._period,
                     granularity=self.granularity)
                 power = self.datasource_backend.statistic_aggregation(
-                    resource_id=source_node.uuid,
-                    meter_name=self.meter_name_power,
+                    resource=source_node,
+                    resource_type='instance',
+                    meter_name='host_power',
                     period=self._period,
                     granularity=self.granularity)
                 if (power < self.threshold_power and
-                        inlet_t < self.threshold_inlet_t):
+                        inlet_temp < self.threshold_inlet_t):
                     # hardware issue, migrate all instances from this node
                     for instance in source_instances:
                         instances_tobe_migrate.append(instance)
@@ -240,6 +178,11 @@ class UniformAirflow(base.BaseStrategy):
                 else:
                     # migrate the first active instance
                     for instance in source_instances:
+                        # NOTE: skip exclude instance when migrating
+                        if instance.watcher_exclude:
+                            LOG.debug("Instance is excluded by scope, "
+                                      "skipped: %s", instance.uuid)
+                            continue
                         if (instance.state !=
                                 element.InstanceState.ACTIVE.value):
                             LOG.info(
@@ -299,27 +242,25 @@ class UniformAirflow(base.BaseStrategy):
         """Group hosts based on airflow meters"""
 
         nodes = self.get_available_compute_nodes()
-        if not nodes:
-            raise wexc.ClusterEmpty()
         overload_hosts = []
         nonoverload_hosts = []
         for node_id in nodes:
             airflow = None
             node = self.compute_model.get_node_by_uuid(
                 node_id)
-            resource_id = node.uuid
             airflow = self.datasource_backend.statistic_aggregation(
-                resource_id=resource_id,
-                meter_name=self.meter_name_airflow,
+                resource=node,
+                resource_type='compute_node',
+                meter_name='host_airflow',
                 period=self._period,
                 granularity=self.granularity)
             # some hosts may not have airflow meter, remove from target
             if airflow is None:
-                LOG.warning("%s: no airflow data", resource_id)
+                LOG.warning("%s: no airflow data", node.uuid)
                 continue
 
             LOG.debug("%(resource)s: airflow %(airflow)f",
-                      {'resource': resource_id, 'airflow': airflow})
+                      {'resource': node, 'airflow': airflow})
             nodemap = {'node': node, 'airflow': airflow}
             if airflow >= self.threshold_airflow:
                 # mark the node to release resources
@@ -329,21 +270,17 @@ class UniformAirflow(base.BaseStrategy):
         return overload_hosts, nonoverload_hosts
 
     def pre_execute(self):
-        LOG.debug("Initializing Uniform Airflow Strategy")
+        self._pre_execute()
+        self.meter_name_airflow = 'host_airflow'
+        self.meter_name_inlet_t = 'host_inlet_temp'
+        self.meter_name_power = 'host_power'
 
-        if not self.compute_model:
-            raise wexc.ClusterStateNotDefined()
-
-        if self.compute_model.stale:
-            raise wexc.ClusterStateStale()
-
-        LOG.debug(self.compute_model.to_string())
-
-    def do_execute(self):
         self.threshold_airflow = self.input_parameters.threshold_airflow
         self.threshold_inlet_t = self.input_parameters.threshold_inlet_t
         self.threshold_power = self.input_parameters.threshold_power
         self._period = self.input_parameters.period
+
+    def do_execute(self, audit=None):
         source_nodes, target_nodes = self.group_hosts_by_airflow()
 
         if not source_nodes:
@@ -381,12 +318,11 @@ class UniformAirflow(base.BaseStrategy):
             destination_node = info['node']
             if self.compute_model.migrate_instance(
                     instance, source_node, destination_node):
-                parameters = {'migration_type': 'live',
-                              'source_node': source_node.uuid,
-                              'destination_node': destination_node.uuid}
-                self.solution.add_action(action_type=self.MIGRATION,
-                                         resource_id=instance.uuid,
-                                         input_parameters=parameters)
+                self.add_action_migrate(
+                    instance,
+                    'live',
+                    source_node,
+                    destination_node)
 
     def post_execute(self):
         self.solution.model = self.compute_model
